////////////////////////////////////////////////////////
// Project  : FHS
//
// File     : fhs.c
// Purpose  :
// Author   : Damir Krstinic
// Copyright: (c) Damir Krstinic
// Version  : v1.0
////////////////////////////////////////////////////////
#include <stdio.h>
#include <stdlib.h>
#include <sys/time.h>
#include <string.h>

#include "fhs.h"
#include <assert.h>

/*
 *
 * MAIN
 *
 */
int main(int argc, char **argv) {
  char *in_imgfile, *segout, filename[256];
  float *hist=NULL;
  // histogram bins in each dimension, complete num.of bins
  int nbins[3], hbins;
  int minpts;
  float hr;
  int segments;  // num.of segments
  IplImage *hlabels;
  
#ifdef TIMES_OUT
  struct timeval time1, time2;
  int sectmp, msectmp;
#endif
  
  if (argc != 5) {
    printf("usage: %s <in_img> <seg_out> <hr> <minpts>\n", argv[0]);
    return 0;
  }
  
  in_imgfile = argv[1];
  segout = argv[2];
  hr = (float)atof(argv[3]);
  minpts = atoi(argv[4]);
  
#ifdef TIMES_OUT
  gettimeofday(&time1, NULL);
#endif

  IplImage *srcimg = cvLoadImage(in_imgfile, CV_LOAD_IMAGE_COLOR);
  if (!srcimg) {
    perror("cvLoadImage");
    exit(1);
  }

  IplImage *wcolimg = rgb2wcol(srcimg);
  if (!wcolimg) {
    fprintf(stderr, "rgb2wcol");
    exit(1);
  }

#ifdef TIMES_OUT
  gettimeofday(&time2, NULL);
  sectmp = time2.tv_sec - time1.tv_sec;
  msectmp = (time2.tv_usec - time1.tv_usec)/1000;
  if (msectmp < 0) {
    sectmp--;
    msectmp += 1000;
  }
  fprintf(stdout, "ppm2luv: \t\t %3d:%03d\n",	\
	  sectmp, msectmp); fflush(stdout);
  
  memcpy(&time1, &time2, sizeof(struct timeval));
#endif
  
#ifdef STD_OUT
  printf("\nimage: %s\n", in_imgfile);
  printf("----------------------------------------------------\n");
  printf("height: \t %4d\n", srcimg->height);
  printf("width:  \t %4d\n", srcimg->width);
  fflush(stdout);
#endif
  
  hist = createAHist3D(wcolimg, &hlabels, hr, nbins);
  if (!hist) {
    fprintf(stderr, "createASH3D error");
    exit(1);
  }
  hbins = nbins[0]*nbins[1]*nbins[2];
  
#ifdef TIMES_OUT
  gettimeofday(&time2, NULL);
  sectmp = time2.tv_sec - time1.tv_sec;
  msectmp = (time2.tv_usec - time1.tv_usec)/1000;
  if (msectmp < 0) {
    sectmp--;
    msectmp += 1000;
  }
  fprintf(stdout, "createAHist3D: \t\t %3d:%03d\n",	\
	  sectmp, msectmp); fflush(stdout);
  
  memcpy(&time1, &time2, sizeof(struct timeval));
#endif
  
#ifdef STD_OUT
  printf("\nHistogram:\n");
  printf("----------------------------------------------------\n");
  printf("nbins: \t %d, %d, %d\n", nbins[0], nbins[1], nbins[2]);
  printf("hbins: \t %d\n", hbins);
  fflush(stdout);
#endif
  
  segments = assignLabels3DBU(hist, nbins, EPS1, EPS0);
  if (segments == -1) {
    fprintf(stderr, "assignLabels3DBU error\n");
    exit(1);
  }
  
#ifdef TIMES_OUT
  gettimeofday(&time2, NULL);
  sectmp = time2.tv_sec - time1.tv_sec;
  msectmp = (time2.tv_usec - time1.tv_usec)/1000;
  if (msectmp < 0) {
    sectmp--;
    msectmp += 1000;
  }
  
  fprintf(stdout, "assignLabels3DBU: \t %3d:%03d\n",	\
	  sectmp, msectmp); fflush(stdout);
  
  memcpy(&time1, &time2, sizeof(struct timeval));
#endif
  
#ifdef STD_OUT
  printf("\nClustering: %d clusters found\n", segments);
  fflush(stdout);
#endif

#ifndef NDEBUG
  int i, j;
  int *hldata;
  for (i=0; i<hlabels->height; i++) {
    hldata = (int *)(hlabels->imageData + i*hlabels->widthStep);
    for (j=0; j<hlabels->width; j++) {
      assert(hldata[j] >= 0 && hldata[j] < hbins);
    }
  }
#endif

  IplImage *seglabels = \
    gridLabels2img(wcolimg, hist, hlabels, &segments, hbins, minpts);
  if (!seglabels) {
    fprintf(stderr, "gridLabels2img error");
    exit(1);
  }
  
#ifdef TIMES_OUT
  gettimeofday(&time2, NULL);
  sectmp = time2.tv_sec - time1.tv_sec;
  msectmp = (time2.tv_usec - time1.tv_usec)/1000;
  if (msectmp < 0) {
    sectmp--;
    msectmp += 1000;
  }
  
  fprintf(stdout, "gridLabels2img: \t %3d:%03d\n",	\
	  sectmp, msectmp); fflush(stdout);
  
  memcpy(&time1, &time2, sizeof(struct timeval));
#endif
  
#ifdef STD_OUT
  printf("\nImage segmented, %d segments found\n", segments);
  fflush(stdout);
#endif
  
  if (seg2img(srcimg, seglabels, segments) == -1) {
    fprintf(stderr, "seg2img error");
    exit(1);
  }
  
#ifdef MARK_EDGES
  mark_edges(srcimg, seglabels);
#endif

  cvSaveImage(segout, srcimg, 0 );
  
  
#ifdef TIMES_OUT
  gettimeofday(&time2, NULL);
  sectmp = time2.tv_sec - time1.tv_sec;
  msectmp = (time2.tv_usec - time1.tv_usec)/1000;
  if (msectmp < 0) {
    sectmp--;
    msectmp += 1000;
  }
  
  fprintf(stdout, "Snimanje rezultata: \t %3d:%03d\n",	\
	  sectmp, msectmp); fflush(stdout);
#endif
  
  cvReleaseImage(&srcimg);
  cvReleaseImage(&wcolimg);
  cvReleaseImage(&seglabels);
  cvReleaseImage(&hlabels);

  free(hist);
  
  return 0;
}
