////////////////////////////////////////////////////////
// Project  : FHS
//
// File     : tools/fhs_dir.c
// Purpose  : FHS segm. of all images in dir
// Author   : Damir Krstinic
// Copyright: (c) Damir Krstinic
// Version  : v1.0
////////////////////////////////////////////////////////
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/time.h>
#include <unistd.h>
#include <dirent.h>
#include <fcntl.h>

#define FHS_BIN "./fhs"

int main(int argc, char *argv[]) {
    DIR *dp;
    struct dirent *dent;
    char infile[256], outfile[256], cmd[512];
    struct stat buf;
    char *indir, *outdir;
    unsigned int nseg = 0;
    struct timeval time1, time2;
    int sec = 0, msec = 0, sectmp, msectmp;
    int fd;
    int minpts, eps1, sc;
    float hr;

    if (argc != 5) {
      printf("usage: %s <in_img_dir> <seg_out_dir> <hr> <MinPts>\n", argv[0]);
	return 0;
    }

    indir = argv[1]; outdir = argv[2];
    hr = atof(argv[3]); 
    minpts = atoi(argv[4]);
    if (hr <= 0 || minpts < 0) {
	fprintf(stderr, "bad arguments!\n");
	return 0;
    }
    
    sprintf(outfile, "%s/FHS_DIR.OUT", outdir);
    fd = open(outfile, O_WRONLY | O_CREAT, S_IRUSR | S_IWUSR);
    if (!fd) {
	perror("open");
	return 1;
    }
    dup2(fd, STDOUT_FILENO);
    dup2(fd, STDERR_FILENO);
    if (fd != STDOUT_FILENO && fd != STDERR_FILENO)
	close(fd);

    fprintf(stdout, "# FHS\n");
    fprintf(stdout, "hr = %f\nMinPts = %d\n", hr, minpts);
    fprintf(stdout, "====================================================\n");
    fflush(stdout);

    dp = opendir(indir);
    if (dp == NULL) {
	perror("opendir");
	return 0;
    }

    while ((dent = readdir(dp)) != NULL) {
	sprintf(infile, "%s/%s", indir, dent->d_name);
	if (stat(infile, &buf) < 0) {
	    perror("stat");
	    continue;
	}

	if (S_ISREG(buf.st_mode)) {
	    nseg++;
	    sprintf(outfile, "%s/%s", outdir, dent->d_name);
	    sprintf(cmd, "%s %s\t %s\t %f %d", FHS_BIN, infile, outfile, \
		    hr, minpts);
	    fprintf(stdout, "%3d. CMD: %s ... ", nseg, cmd); 

	    gettimeofday(&time1, NULL);
	    if (system(cmd) == -1) {
		perror("system");
		printf("error on: %s\n", cmd);
	    }
	    gettimeofday(&time2, NULL);
	    sectmp = time2.tv_sec - time1.tv_sec;
	    msectmp = (time2.tv_usec - time1.tv_usec)/1000;
	    if (msectmp < 0) {
		sectmp--;
		msectmp += 1000;
	    }
	    fprintf(stdout, "done\t %3d:%03d\n", sectmp, msectmp); 
	    fflush(stdout);
	    sec += sectmp;
	    msec += msectmp;
	}
    }
  
    closedir(dp);
   
    sec += msec/1000;
    msec = msec%1000;

    fprintf(stdout, "====================================================\n");

    fprintf(stdout, "time sum \t %6d:%03d\n", sec, msec);
    msec = (double)msec/nseg + ((double)sec/nseg - sec/nseg)*1000;
    sec = sec/nseg;
    sec += msec/1000;
    msec = msec%1000;
    fprintf(stdout, "time mean\t %6d:%03d\n", sec, msec); 
    return 0;
}
