////////////////////////////////////////////////////////
// Project  : FHS
//
// File     : utils.c
// Purpose  : Segmentation utils
// Author   : Damir Krstinic
// Copyright: (c) Damir Krstinic
// Version  : v1.0
////////////////////////////////////////////////////////
#include <stdio.h>

#include "fhs.h"

#include <assert.h>

int seg2img(IplImage *srcimg, IplImage *seglabels, int segments) {
  int i, j;
  int k, k4;
  double *segcolors;
  int npix = srcimg->width * srcimg->height;
  
  uchar *idata;
  int *sdata;

  if (segments < 2) {
    cvZero(srcimg);
    return 0;
  }
  
  if (!(segcolors = calloc(4*segments, sizeof(double)))) {
    perror("calloc");
    return -1;
  }
  
  for (i=0; i<srcimg->height; i++) {
    idata = srcimg->imageData + i*srcimg->widthStep;
    sdata = (int *)(seglabels->imageData + i*seglabels->widthStep);
    for (j=0; j<srcimg->width; j++, idata += srcimg->nChannels) {
      assert(sdata[j] < segments);
      
      segcolors[sdata[j]*4]     += idata[0];
      segcolors[sdata[j]*4 + 1] += idata[1];
      segcolors[sdata[j]*4 + 2] += idata[2];
      segcolors[sdata[j]*4 + 3] += 1;
    }
  }
  
  for (k=0; k<segments; k++) {
    k4 = k*4;
    segcolors[k4] = segcolors[k4]/segcolors[k4 + 3];
    segcolors[k4 + 1] = segcolors[k4+1]/segcolors[k4 + 3];
    segcolors[k4 + 2] = segcolors[k4+2]/segcolors[k4 + 3];
  }
  
  for( i = 0 ; i < srcimg->height ; i++ ) {
    idata = srcimg->imageData + i*srcimg->widthStep;
    sdata = (int *)(seglabels->imageData + i*seglabels->widthStep);
    for( j = 0 ; j < srcimg->width ; j++, idata += srcimg->nChannels) {
      assert(sdata[j] < segments);
      idata[0]   = (float)segcolors[sdata[j]*4];
      idata[1] = (float)segcolors[sdata[j]*4+1];
      idata[2] = (float)segcolors[sdata[j]*4+2];
    }
  }
  free(segcolors);
  return 0;
}

IplImage *rgb2wcol(IplImage *srcimg) {
  IplImage *wcolimg = cvCreateImage(cvSize(srcimg->width, srcimg->height), \
				    IPL_DEPTH_32F, srcimg->nChannels);
  if (!wcolimg) {
    perror("cvCreateImage");
    return NULL;
  }

#ifdef LUV_CSPACE
  IplImage *tmp = cvCreateImage(cvSize(srcimg->width, srcimg->height), \
				    IPL_DEPTH_32F, srcimg->nChannels);
  if (!tmp) {
    perror("cvCreateImage");
    return NULL;
  }
  cvConvertScale(srcimg, tmp, 1.0/255.0, 0);
  cvCvtColor(tmp, wcolimg, CV_BGR2Luv);
  cvReleaseImage(&tmp);
#else   // BGR wcolor
  cvConvert(srcimg, wcolimg);
#endif

  return wcolimg;
}

//
// mark_edges
//


void mark_edges(IplImage *img, IplImage *seg) {
  int i, j;
  uchar *idata;
  int *srow, *sup;
 
  for (i=0; i<img->height; i++) {
    idata = img->imageData + i*img->widthStep;
    srow = (int *)(seg->imageData + i*seg->widthStep);
    if (i>0)
      sup = (int *)(seg->imageData + (i-1)*seg->widthStep);
    else
      sup = NULL;
 
    for (j=0; j<img->width; j++) {
      if (j > 0) {
	if (srow[j] != srow[j-1]) {
	  idata[j*img->nChannels] = idata[j*img->nChannels+1] = \
	    idata[j*img->nChannels+2] = 255;
	}
      }
      if (sup) {
	if (srow[j] != sup[j]) {
	  idata[j*img->nChannels] = idata[j*img->nChannels+1] = \
	    idata[j*img->nChannels+2] = 255;
	} 
      }
    }
  }
}
